#import gettext
from snack import *
from constants_text import *
from translate import _
import string
import iutil
from log import log
    
#cat = gettext.Catalog ("anaconda", "/usr/share/locale")
#_ = cat.gettext

class YabootAppendWindow:

    def __call__(self, screen, todo):
	#if not todo.fstab.setupFilesystems or todo.fstab.rootOnLoop():
	#    todo.skipYaboot = 1
	#    return INSTALL_NOOP
        #else:
        #    todo.skipYaboot = 0

        if todo.yaboot.machineType == 'iSeries':
            todo.skipYaboot = 1;
            return INSTALL_NOOP
        else:
            todo.skipYaboot = 0
            
        self.bootMethodWindow(screen, todo)
        
        if not todo.yaboot.getBootMethod() == 'yaboot':
            todo.skipYaboot = 1
            todo.yaboot.setDevice(None)
            return INSTALL_OK

	t = TextboxReflowed(53,
		     _("A few systems will need to pass special options "
		       "to the kernel at boot time for the system to function "
		       "properly. If you need to pass boot options to the "
		       "kernel, enter them now. If you don't need any or "
		       "aren't sure, leave this blank."))

	entry = Entry(48, scroll = 1, returnExit = 1)
	if todo.yaboot.getAppend():
	    entry.set(todo.yaboot.getAppend())

	buttons = ButtonBar(screen, [(_("OK"), "ok"), (_("Skip"), "skip"),  
			     (_("Back"), "back") ] )

	grid = GridFormHelp(screen, _("Yaboot Configuration"), "yabootkernelopts", 1, 4)
	grid.add(t, 0, 0, padding = (0, 0, 0, 1))

	grid.add(entry, 0, 2, padding = (0, 0, 0, 1))
	grid.add(buttons, 0, 3, growx = 1)

        result = grid.runOnce ()
        button = buttons.buttonPressed(result)
        
        if button == "back":
            return INSTALL_BACK

	#if button == "skip":
	#    todo.skipYaboot = 1
        #    todo.yaboot.setDevice(None)
	#else:
	#    todo.skipYaboot = 0

	if entry.value():
	    todo.yaboot.setAppend(string.strip(entry.value()))
	else:
	    todo.yaboot.setAppend(None)

	return INSTALL_OK

    ##################################################################
    # bootMethod()
    #
    #  find out how the user wants to boot the system
    #  they can select which PReP partition to boot from later
    #
    def bootMethodWindow (self, screen, todo):
        t = TextboxReflowed(53,
                            _(""
                              "How will you be booting your system?"
                              ""))

	buttons = ButtonBar(screen, [(_("OK"), "ok"), (_("Back"), "back") ] )

        # XXX write a help text for this window!!!
	grid = GridFormHelp(screen, _("Boot Configuration"), "ppcbootmethod", 1, 4)
	listboxLabel = Label("%-15s" % ( _("Boot Method")))
	listbox = Listbox(3, scroll = 1, returnExit = 1)


        bootOptions = [[_("Boot kernel using the Yaboot bootloader"), 'yaboot'],
                       [_("Boot kernel directly from PReP partition"), 'kernel'],
                       [_("Do not configure a boot method"), None]]

        for (method, key) in bootOptions:
            listbox.append(("%-32s" % (method)), key)

	grid.add(t, 0, 0, padding = (0, 0, 0, 1))

	grid.add(listboxLabel, 0, 1, padding = (0, 1, 0, 0), anchorLeft = 0)
	grid.add(listbox, 0, 2, padding = (0, 0, 0, 1), anchorLeft = 0)

	grid.add(buttons, 0, 3, growx = 1)

        result = grid.runOnce ()
        button = buttons.buttonPressed(result)
        
        if button == "back":
            return INSTALL_BACK

        todo.yaboot.setBootMethod(listbox.current())

class YabootWindow:
    
    def __call__(self, screen, todo):
#        if not todo.setupFilesystems: return INSTALL_NOOP
	(mount, dev, fstype, format, size) = todo.fstab.mountList()[0]
	if mount != '/': return INSTALL_NOOP

        # todo.skipYaboot == 1 is OK since we use this for kernels also
	if not todo.yaboot.getBootMethod(): return INSTALL_NOOP

	if not todo.yaboot.allowYabootLocationConfig(todo.fstab):
	    return INSTALL_NOOP

        plist = []
        plist = todo.yaboot.getPrepPartitionList(todo.fstab)

        if len( plist ) < 1:
            todo.skipYaboot = 1
            todo.yaboot.setDevice(None)
            return INSTALL_OK

        if todo.yaboot.getBootMethod() == 'kernel':
            bootMedia = 'The kernel'
            helpText = "kernellocation"
        else:
            bootMedia = 'Yaboot'
            helpText = "yabootlocation"
            
	t = TextboxReflowed(53,
		     _(bootMedia + " must be installed in a PReP boot "
                       "partition. You can then set open firmware to "
                       "boot that device. "
                       "\n\n "
                       "Select the PReP partition onto which you would "
                       "like to install " + bootMedia + ". "
                       ""))

	buttons = ButtonBar(screen, [(_("OK"), "ok"), (_("Back"), "back") ] )

	grid = GridFormHelp(screen, _("Boot Configuration"), helpText, 1, 4)
	listboxLabel = Label("%-10s" % ( _("Device")))
	listbox = Listbox(len(plist), scroll = 1, returnExit = 1)

	for dev in plist:
	    listbox.append(("%-10s" % dev), dev)

        listbox.setCurrent(plist[0])

	grid.add(t, 0, 0, padding = (0, 0, 0, 1))

	grid.add(listboxLabel, 0, 1, padding = (0, 1, 0, 0), anchorLeft = 0)
	grid.add(listbox, 0, 2, padding = (0, 0, 0, 1), anchorLeft = 0)

	grid.add(buttons, 0, 3, growx = 1)

        result = grid.runOnce ()
        button = buttons.buttonPressed(result)
        
        if button == "back":
            return INSTALL_BACK

	if button == "skip":
            # never happens 
	    todo.skipYaboot = 1
            todo.yaboot.setDevice(None)
	else:
            dev = listbox.current()
            todo.yaboot.setDevice(dev)

        return INSTALL_OK
        
##############################################################################
   
class YabootImagesWindow:
    ##
    ## this checks for illegal chars in an image label
    ##
    ## what exactly are the rules for yaboot?
    ##
    def validYabootLabel(self, label):
        i=0
        while i < len(label):
            cur = label[i]
            if cur == ' ' or cur == '#' or cur == '$' or cur == '=':
                return 0
            i = i + 1

        return 1

    
    def editItem(self, screen, partition, itemLabel, allowNone=0):
	devLabel = Label(_("Device") + ":")
	bootLabel = Label(_("Boot label") + ":")
	device = Label("/dev/" + partition)
        newLabel = Entry (20, scroll = 1, returnExit = 1, text = itemLabel)

	buttons = ButtonBar(screen, [(_("Ok"), "ok"), (_("Clear"), "clear"),
			    (_("Cancel"), "cancel")])

	subgrid = Grid(2, 2)
	subgrid.setField(devLabel, 0, 0, anchorLeft = 1)
	subgrid.setField(device, 1, 0, padding = (1, 0, 0, 0), anchorLeft = 1)
	subgrid.setField(bootLabel, 0, 1, anchorLeft = 1)
	subgrid.setField(newLabel, 1, 1, padding = (1, 0, 0, 0), anchorLeft = 1)

	g = GridFormHelp(screen, _("Edit Boot Label Please"), "bootlabel", 1, 2)
	g.add(subgrid, 0, 0, padding = (0, 0, 0, 1))
	g.add(buttons, 0, 1, growx = 1)

	result = ""
	while (result != "ok" and result != "F12" and result != newLabel):
	    result = g.run()

	    if (buttons.buttonPressed(result)):
		result = buttons.buttonPressed(result)

	    if (result == "cancel"):
		screen.popWindow ()
		return itemLabel
	    elif (result == "clear"):
		newLabel.set("")
            elif (result == "ok" or result == "F12" or result == newLabel):
		if not allowNone and not newLabel.value():
                    rc = ButtonChoiceWindow (screen, _("Invalid Boot Label"),
                                             _("Boot label may not be empty."),
                                             [ _("OK") ])
                    result = ""
                elif not self.validYabootLabel(newLabel.value()):
                    rc = ButtonChoiceWindow (screen, _("Invalid Boot Label"),
                                             _("Boot label contains "
                                               "illegal characters."),
                                             [ _("OK") ])
                    result = ""

	screen.popWindow()

	return newLabel.value()

    def formatDevice(self, type, label, device, default):
	if (type == 2):
	    type = "Linux Native"
	elif (type == 1):
	    type = "DOS/Windows"
	elif (type == 4):	
	    type = "OS/2 / Windows NT"
	else:
	    type = "Other"

	if default == device:
	    default = '*'
	else:
	    default = ""
	    
	return "%-10s  %-25s %-7s %-10s" % ( "/dev/" + device, type, default, label)

    def __call__(self, screen, todo):
	(images, default) = todo.yaboot.getYabootImages(todo.fstab)
	if not images: return INSTALL_NOOP
	if todo.skipYaboot: return INSTALL_NOOP

	# the default item is kept as a label (which makes more sense for the
	# user), but we want our listbox "indexed" by device, so we keep
	# the default item as a device
        #
	for (dev, (label, type)) in images.items():
	    if label == default:
		default = dev
		break

	sortedKeys = images.keys()
	sortedKeys.sort()

	listboxLabel = Label("%-10s  %-25s %-7s %-10s" % 
		( _("Device"), _("Partition type"), _("Default"), _("Boot label")))
	listbox = Listbox(5, scroll = 1, returnExit = 1)

	for n in sortedKeys:
	    (label, type) = images[n]
	    listbox.append(self.formatDevice(type, label, n, default), n)
	    if n == default:
		listbox.setCurrent(n)

	buttons = ButtonBar(screen, [ (_("Ok"), "ok"), (_("Edit"), "edit"), 
				      (_("Back"), "back") ] )

	text = TextboxReflowed(55,
		    _("The boot manager Red Hat uses can boot other " 
		      "operating systems as well. You need to tell me " 
		      "what partitions you would like to be able to boot " 
		      "and what label you want to use for each of them."))

	g = GridFormHelp(screen, _("Yaboot Configuration"), "lilolabels", 1, 4)
	g.add(text, 0, 0, anchorLeft = 1)
	g.add(listboxLabel, 0, 1, padding = (0, 1, 0, 0), anchorLeft = 1)
	g.add(listbox, 0, 2, padding = (0, 0, 0, 1), anchorLeft = 1)
	g.add(buttons, 0, 3, growx = 1)
	g.addHotKey(" ")

	result = None
        (rootdev, rootfs) = todo.fstab.getRootDevice()
	while (result != "ok" and result != "back" and result != "F12"):
	    result = g.run()
	    if (buttons.buttonPressed(result)):
		result = buttons.buttonPressed(result)

	    if (result == string.lower(_("Edit")) or result == listbox):
		item = listbox.current()
		(label, type) = images[item]

		label = self.editItem(screen, item, label, allowNone = (rootdev != item and item != default))
		images[item] = (label, type)
		if (default == item and not label):
		    default = ""
		listbox.replace(self.formatDevice(type, label, item, default), item)
		listbox.setCurrent(item)
#	    elif result == "F2":
	    elif result == " ":
		item = listbox.current()
		(label, type) = images[item]
		if (label):
		    if (default):
			(oldLabel, oldType) = images[default]
			listbox.replace(self.formatDevice(oldType, oldLabel, default, 
					""), default)
		    default = item
		    listbox.replace(self.formatDevice(type, label, item, default), 
				    item)
		    listbox.setCurrent(item)

	screen.popWindow()

	if (result == "back"):
	    return INSTALL_BACK

	# find the label for the default device
	for (dev, (label, type)) in images.items():
	    if dev == default:
		default = label
		break

	todo.yaboot.setYabootImages(images)
	todo.yaboot.setDefault(label)

	return INSTALL_OK

