import string
import os
import isys
import iutil
import rpm
from translate import _
from log import log

#
# "/proc/e820info" exists only in i386 kernel code!!!
#
def needsEnterpriseKernel():
    return 0

class YabootConfigFile:
    def __repr__ (self, tab = 0):
	s = ""
	for n in self.order:
	    if (tab):
		s = s + '\t'
	    if n[0] == '#':
		s = s + n[1:]
	    else:
		s = s + n
		if self.items[n]:
		    s = s + "=" + self.items[n]
	    s = s + '\n'
	for cl in self.images:
	    s = s + "\n%s=%s\n" % (cl.imageType, cl.path)
	    s = s + cl.__repr__(1)
	return s

    def addEntry(self, item, val = None, replace = 1):
	if not self.items.has_key(item):
	    self.order.append(item)
	elif not replace:
	    return

	if (val):
	    self.items[item] = str(val)
	else:
	    self.items[item] = None

    def getEntry(self, item):
	return self.items[item]

    def delEntry(self, item):
	newOrder = []
	for i in self.order:
	    if item != i: newOrder.append(i)
	self.order = newOrder

	del self.items[item]

    def testEntry(self, item):
        if self.items.has_key(item):
            return 1
        else:
            return 0

    def getImage(self, label):
        for config in self.images:
	    if config.getEntry('label') == label:
		return (config.imageType, config)

	raise IndexError, "unknown image %s" % (label,)

    def addImage (self, config):
	# make sure the config has a valid label
	config.getEntry('label')
	if not config.path or not config.imageType:
	    raise ValueError, "subconfig missing path or image type"

	self.images.append(config)

    def delImage (self, label):
        for config in self.images:
	    if config.getEntry('label') == label:
                self.images.remove (config)
		return

	raise IndexError, "unknown image %s" % (label,)

    def listImages (self):
	l = []
        for config in self.images:
	    l.append(config.getEntry('label'))
	return l

    def getPath (self):
	return self.path

    def write(self, file, perms = 0644):
	f = open(file, "w")
	f.write(self.__repr__())
	f.close()
	os.chmod(file, perms)

    def read (self, file):
	f = open(file, "r")
	image = None
	for l in f.readlines():
	    l = l[:-1] 
	    orig = l
	    while (l and (l[0] == ' ' or l[0] == '\t')):
		l = l[1:]
	    if not l:
		continue

	    if l[0] == '#':
		self.order.append('#' + orig)
		continue
	    fields = string.split(l, '=', 1)
	    if (len(fields) == 2):
		f0 = string.strip (fields [0])
		f1 = string.strip (fields [1])
		if (f0 == "image" or f0 == "other"):
		    if image: self.addImage(image)
		    image = YabootConfigFile(imageType = f0, 
					   path = f1)
		    args = None
                else:
		    args = (f0, f1)
	    else:
		args = (string.strip (l),)

	    if (args and image):
		apply(image.addEntry, args)
	    elif args:
		apply(self.addEntry, args)

	if image: self.addImage(image)
	    
	f.close()

    def __init__(self, imageType = None, path = None):
	self.imageType = imageType
	self.path = path
	self.order = []
	self.images = []
	self.items = {}

class YabootConfiguration:

    def allowYabootLocationConfig(self, fstab):
	bootDevice = fstab.getBootDevice()
	if bootDevice[0:2] == "md":
	    self.setDevice(("raid", bootDevice))
	    return None

	return 1

    def setYabootImages(self, images):
	self.yabootImages = images

    def getYabootImages(self, fstab):
        (drives, raid) = fstab.raidList()

	# rearrange the fstab so it's indexed by device
	mountsByDev = {}
	for (mntpoint, device, fsystem, doFormat, size) in \
		    fstab.mountList():
	    mountsByDev[device] = mntpoint

	for (mntpoint, device, fstype, raidType, start, size, makeup) in raid:
	    mountsByDev[device] = mntpoint
	    drives.append(device, "", 2, 0, 0, 0, 0)

	for (device, mntpoint, fsystem, makeup) in fstab.existingRaidList():
	    mountsByDev[device] = mntpoint
	    drives.append(device, "", 2, 0, 0, 0, 0)
	    
	oldImages = {}
	for dev in self.yabootImages.keys():
	    oldImages[dev] = self.yabootImages[dev]

	self.yabootImages = {}
	for (dev, devName, fsType, start, size, maxcyl, preexist) in drives:
	    # ext2 and raid partitions get listed if they're / -- we do
	    # not allow non-mounted partitions to be booted anymore as
	    # modules are so unlikely to work out as to be not worth
	    # worrying about
	    #
	    # there is a good chance we should configure them as chain
	    # loadable, but we don't

            # only list ext2 partitions
            if fsType != 2:
                continue

	    if (mountsByDev.has_key(dev)):
		if mountsByDev[dev] == '/':
		    if oldImages.has_key(dev):
			self.yabootImages[dev] = oldImages[dev]
		    else:
			self.yabootImages[dev] = ("linux", 2)

	# if there is no default image (self.default is None, or invalid)
	# set the default image to the linux root partition
	if self.default:
	    for (label, fsType) in self.yabootImages.values():
		if label == self.default: break
	    if label != self.default:
		self.default = None

	if not self.default:
	    for (label, fsType) in self.yabootImages.values():
		if fsType == 2:
		    self.default = label
		    break

	return (self.yabootImages, self.default)

    def makeInitrd (self, kernelTag, instRoot):
        initrd = "/boot/initrd%s.img" % (kernelTag, )
        
        # iSeries initrds cannot be compressed
        if os.access('/proc/iSeries', os.R_OK): 
            if not self.initrdsMade.has_key(initrd):
                iutil.execWithRedirect("/sbin/mkinitrd",
                                      [ "/sbin/mkinitrd",
                                        "--ifneeded",
                                        "--nocompress",
                                        initrd,
                                        kernelTag[1:] ],
                                      stdout = None, stderr = None, searchPath = 1,
                                      root = instRoot)
                self.initrdsMade[kernelTag] = 1
        else:
            if not self.initrdsMade.has_key(initrd):
                iutil.execWithRedirect("/sbin/mkinitrd",
                                      [ "/sbin/mkinitrd",
                                        "--ifneeded",
                                        initrd,
                                        kernelTag[1:] ],
                                      stdout = None, stderr = None, searchPath = 1,
                                      root = instRoot)
                self.initrdsMade[kernelTag] = 1
                
	return initrd

    def iSeriesBootSetup(self, fstab, instRoot, hdList, upgrade):
	
        # 1. find out the name of the device containing "/"
        rootDev = fstab.getRootDevice ()[0]
                    
        if rootDev[0:2] == "vd":
            rootDev = '/dev/iseries/' + rootDev
        else:
            rootDev = '/dev/' + rootDev
            
        # 2. see whether we have installed an smp kernel on this machine
        smpInstalled = (hdList.has_key('kernel-smp') and 
                        hdList['kernel-smp'].selected)
                            
        kernelInstalled = (hdList.has_key('kernel-iseries') and 
                           hdList['kernel-iseries'].selected)
    
        if (upgrade and not isys.smpAvailable()):
            smpInstalled = 0
    
        # 3. choose the appropriate kernel to boot when install is done
        kernel = None
        if (smpInstalled):
            kernel = hdList['kernel-smp']
            tag = "smp"
        elif kernelInstalled:
            kernel = hdList['kernel-iseries']
            tag = ""
    
        if kernel is None:
            return
                
        # 4. formulate the filename for the chosen kernel
        kernelTag = "-%s-%s%s" % (kernel[rpm.RPMTAG_VERSION],
				  kernel[rpm.RPMTAG_RELEASE],
                                  tag)
    
        kernelFile = "/boot/vmlinux" + kernelTag

	# 5. make an initrd (not compressed) so we can boot next time
        initrd = self.makeInitrd (kernelTag, instRoot)

	sysMap = "/boot/System.map" + kernelTag

	#	    @side@ : (@kernel@, @initrd@, @cmdline@, @exitOnFailure@)
	slotInfo = {"C" : (kernelFile, initrd, "root=" + rootDev, 1),
		    "A" : (kernelFile, "/boot/initrd-install.img", "expert rescue", 0),
		    "B" : (kernelFile, initrd, "root=" + rootDev, 1) }

	for (side, (kernelFile, initrd, cmdline, abort)) in slotInfo.items():
	    if os.access (instRoot + initrd, os.R_OK) and \
	       os.access('/sbin/addRamDisk', os.X_OK):
		try:
		    # 6. piggyback the initrd on the kernel
		    iutil.execWithRedirect('/sbin/addRamDisk',
					   ["addRamDisk",
					    instRoot + initrd,
					    instRoot + sysMap,
					    instRoot + kernelFile,
                                            instRoot + kernelFile + '.initrd'],
					   stdout = None,
                                           stderr = None)
		    kernelFile = kernelFile + '.initrd'
		except:
		    if abort:
			return "Failed to exec /sbin/addRamDisk"
		    else:
			continue

	    try:
		# 7. dd this nasty ho into @side@ to be booted next
		iutil.execWithRedirect(instRoot + '/bin/dd',
				       ["dd",
					"if=" + instRoot +  kernelFile,
					"of=/proc/iSeries/mf/%s/vmlinux" % side,
					"bs=32k" ],
					stdout = None,
					stderr = None)
	    except:
		if abort:
		    return "Failed to install kernel onto %s side" % side
		else:
		    continue
                
	    try:
		# 8. clear the command line on the "C side" from any install args
		iutil.execWithRedirect(instRoot + '/bin/dd',
				       ["dd",
					"if=" + instRoot + "/dev/zero",
					"of=/proc/iSeries/mf/%s/cmdline" % side,
					"bs=256",
					"count=1" ],
				       stdout = None,
                                       stderr = None)
	    except:
		if abort:
		    return "Failed to clear install command line args"
		else:
		    continue

	    try:
		# 9. supply the 'root=' command line for the next reboot
		iutil.execWithRedirect(instRoot + '/bin/echo',
				       ["echo",
					cmdline ],
                                       stdout = "/proc/iSeries/mf/%s/cmdline" % side,
				       stderr = None)
	    except:
		if abort:
		    return "Failed to set %s side command line" % side
		else:
		    continue

    def install(self, fstab, instRoot, hdList, upgrade):
	
        if self.machineType == 'iSeries':
            return self.iSeriesBootSetup(fstab, instRoot, hdList, upgrade)

        """
            OK. This is where pSeries stuff begins
        """
        # If self.yabootDevice is None, skipping yaboot doesn't work
	if not self.yabootDevice:
            return

	# yeck. setting yabootDevice to "partition" means auto-setup (ks)
	if self.yabootDevice == "partition":
	    plist = self.getPrepPartitionList(fstab)
	    if len(plist):
		self.setDevice(plist[0])
		self.setBootMethod("yaboot")

	# If the root partition is on a loopback device, yaboot won't work!
	if fstab.rootOnLoop():
	    return 

        # handle 'kernel' and 'noBoot' bootMethod cases first
        if not self.getBootMethod():
            return
        elif self.getBootMethod() == 'kernel':
            smpInstalled = (hdList.has_key('kernel-smp') and 
                            hdList['kernel-smp'].selected)

            kernelInstalled = (hdList.has_key('kernel-pseries') and 
                               hdList['kernel-pseries'].selected)
                               
            # This is a bit odd, but old versions of Red Hat could install
            # SMP kernels on UP systems, but (properly) configure the UP version.
            # We don't want to undo that, but we do want folks using this install
            # to be able to override the kernel to use during installs. This rule
            # seems to nail this.
            if (upgrade and not isys.smpAvailable()):
                smpInstalled = 0
    
            kernel = None
            if (smpInstalled):
                kernel = hdList['kernel-smp']
                tag = "smp"
            elif kernelInstalled:
                kernel = hdList['kernel-pseries']
                tag = ""

            if kernel is None:
                return
                
            kernelTag = "-%s-%s%s" % (kernel[rpm.RPMTAG_VERSION],
                                    kernel[rpm.RPMTAG_RELEASE],
                                    tag)
            kernelFile = "/boot/vmlinux" + kernelTag

            iutil.execWithRedirect(instRoot + '/bin/dd',
                                   [ "dd",
                                     "if=" + instRoot + kernelFile,
                                     "of=" + self.yabootDevice ],
                                   stdout = None,
                                   stderr = None)
            return

	if not self.yabootImages:
	    (images, default) = self.getYabootImages(fstab)
	    self.setYabootImages(images)

        # on upgrade read in the yaboot config file
	yaboot = YabootConfigFile ()
	perms = 0644

        if os.access (instRoot + '/etc/yaboot.conf', os.R_OK):
	    perms = os.stat(instRoot + '/etc/yaboot.conf')[0] & 0777
            # XXX just ignore the yaboot.conf that comes in the yaboot RPM for now...
	    #yaboot.read (instRoot + '/etc/yaboot.conf')
	    os.rename(instRoot + '/etc/yaboot.conf',
		      instRoot + '/etc/yaboot.conf.rpmsave')

	# Remove any invalid entries that are in the file; we probably
	# just removed those kernels. 
	for label in yaboot.listImages():
	    (fsType, sl) = yaboot.getImage(label)
	    if fsType == "other": continue

	    if not os.access(instRoot + sl.getPath(), os.R_OK):
		yaboot.delImage(label)                

        # figure out which partition /boot is on
        bootPartition = fstab.getBootDevice()
        
        yabootTarget = self.yabootDevice
        initMsg = \
                r'"\nWelcome to Red Hat Linux!\nHit <TAB> for boot options.\n\n"'

        yaboot.addEntry("boot", yabootTarget)
        yaboot.addEntry("partition", bootPartition)
        yaboot.addEntry("init-message", initMsg)
	yaboot.addEntry("timeout", "50", replace=0)
        yaboot.addEntry("install", "/usr/lib/yaboot/yaboot")
        yaboot.addEntry("nonvram", None, replace=0)
        
	smpInstalled = (hdList.has_key('kernel-smp') and 
                        hdList['kernel-smp'].selected)

        kernelInstalled = (hdList.has_key('kernel-pseries') and 
                           hdList['kernel-pseries'].selected)
                        
        if not smpInstalled and not kernelInstalled:
            return
            
	# This is a bit odd, but old versions of Red Hat could install
	# SMP kernels on UP systems, but (properly) configure the UP version.
	# We don't want to undo that, but we do want folks using this install
	# to be able to override the kernel to use during installs. This rule
	# seems to nail this.
	if (upgrade and not isys.smpAvailable()):
	    smpInstalled = 0

        rootDev = fstab.getRootDevice ()
        if rootDev:
	    # strip off the filesystem; we don't need it
            rootDev = rootDev[0]
        else:
            raise RuntimeError, "Installing yaboot, but there is no root device"

        kernelList = []
        otherList = []

        main = self.default

        for (drive, (label, yabootType)) in self.yabootImages.items ():
            if (drive == rootDev) and label:
                main = label
            elif label:
                otherList.append (label, "/dev/" + drive)

	yaboot.addEntry("default", self.default)

	mainLabelUsed = 0

	if (smpInstalled):
	    thisLabel = main
	    if mainLabelUsed:
		thisLabel = thisLabel + '-smp'
	    mainLabelUsed = 1

	    kernelList.append((thisLabel, hdList['kernel-smp'], "smp"))

	thisLabel = main
	if mainLabelUsed:
	    thisLabel = thisLabel + '-up'

	kernelList.append((thisLabel, hdList['kernel-pseries'], ""))

	for (label, kernel, tag) in kernelList:
	    kernelTag = "-%s-%s%s" % (kernel[rpm.RPMTAG_VERSION],
                                      kernel[rpm.RPMTAG_RELEASE], tag)
	    kernelFile = "/boot/vmlinux" + kernelTag

	    try:
		(fsType, sl) = yaboot.getImage(label)
		yaboot.delImage(label)
	    except IndexError, msg:
		sl = YabootConfigFile(imageType = "image", path = kernelFile)

	    initrd = self.makeInitrd (kernelTag, instRoot)

	    sl.addEntry("label", label)
            
            #
            # FIXME: yaboot currently cannot boot a vmlinux w/ initrd (16 July 2002)
            #
            #   (it can boot vmlinux w/o initrd or zImage.initrd)
            #
	    if os.access (instRoot + initrd, os.R_OK) and os.uname()[4] != "ppc64":
		sl.addEntry("initrd", initrd)

	    sl.addEntry("read-only")
	    sl.addEntry("root", '/dev/' + rootDev)

	    if self.yabootAppend:
		sl.addEntry('append', '"%s"' % (self.yabootAppend,))
		
	    yaboot.addImage (sl)

	for (label, device) in otherList:
	    try:
		(fsType, sl) = yaboot.getImage(label)
		yaboot.delImage(label)
	    except IndexError:
		sl = YabootConfigFile(imageType = "other", path = device)
		sl.addEntry("optional")

	    sl.addEntry("label", label)
	    yaboot.addImage (sl)

	# Sanity check #1. There could be aliases in sections which conflict
	# with the new images we just created. If so, erase those aliases
	imageNames = {}
	for label in yaboot.listImages():
	    imageNames[label] = 1

	for label in yaboot.listImages():
	    (fsType, sl) = yaboot.getImage(label)
	    if sl.testEntry('alias'):
		alias = sl.getEntry('alias')
		if imageNames.has_key(alias):
		    sl.delEntry('alias')
		imageNames[alias] = 1

	# Sanity check #2. If single-key is turned on, go through all of
	# the image names (including aliases) (we just built the list) and
	# see if single-key will still work.
	if yaboot.testEntry('single-key'):
	    singleKeys = {}
	    turnOff = 0
	    for label in imageNames.keys():
		l = label[0]
		if singleKeys.has_key(l):
		    turnOff = 1
		singleKeys[l] = 1
	    if turnOff:
		yaboot.delEntry('single-key')
        
	yaboot.write(instRoot + "/etc/yaboot.conf", perms = perms)
        
        str = iutil.execWithCapture("/usr/sbin/mkofboot",
                              ["mkofboot",
                               "--filesystem",
                               "raw",
                               "--force"],
                              root=instRoot,
                              catchfd=2)

        return str
        
    def getPrepPartitionList(self, fstab):
        plist = []
	(drives, raid) = fstab.partitionList()
        rootDev = fstab.getRootDevice()[0]
	for (dev, devName, type, start, size, maxcyl, preexist) in drives:
            if type == 8:
                if (dev[:3] != rootDev[:3]) and (rootDev[:2] != "md"):
                    continue
                if os.access("/proc/iSeries", os.R_OK):
                    device = "/dev/iseries/" + dev
                else:
                    device = '/dev/' + dev 

                log("found PReP partition -> %s\n", device)
                plist.append(device)
        return plist

    def setDevice(self, device=None):
	self.yabootDevice = device

    def setBootMethod(self, method=None):
        self.bootMethod = method

    def setAppend(self, append):
	self.yabootAppend = append

    def setDefault(self, default):
	for (label, fsType) in self.yabootImages.values():
	    if label == default:
		self.default = default
		return
	raise IndexError, "unknown yaboot label %s" % (default,)

    def getDevice(self):
	return self.yabootDevice

    def getAppend(self):
	return self.yabootAppend

    def getBootMethod(self):
        return self.bootMethod

    def getMachineType(self):
        return self.machineType
    
    def __init__(self):
	self.yabootImages = {}
	self.yabootDevice = None
	self.yabootAppend = None
	self.default = None
	self.initrdsMade = {}
        self.edd = 0
        self.bootMethod = None
        
        if os.access("/proc/iSeries", os.R_OK):
            self.machineType = 'iSeries'
        else:
            self.machineType = 'pSeries'	        

if __name__ == "__main__":
    config = YabootConfigFile ()
    config.read ('/etc/yaboot.conf')
    print config
    print "image list", config.listImages()
    config.delImage ('linux')
    print '----------------------------------'
    config = YabootConfigFile ()
    config.read ('/etc/yaboot.conf')
    print config
    print '----------------------------------'    
    print config.getImage('linux')
